from rest_framework.test import APIClient
from datetime import datetime

from django.utils import timezone 

class OperisTestClient(APIClient):

    def get_iteration_string(self,iteration=None):
        if not iteration:
            return ""
        return str(iteration)
        
    def generate_test_data(self,fields,iteration=None,omit_id=True):
        item = {}
        for field in fields:
            if field['name'] == 'id' and omit_id:
                continue
            item[field['name']] = self.generate_field_data(field,iteration)
        return item
            
    def generate_field_data(self,field,iteration=None):
        """
        'AutoField', 'BLANK_CHOICE_DASH', 'BigIntegerField', 'BinaryField',
        'BooleanField', 'CharField', 'CommaSeparatedIntegerField', 'DateField',
        'DateTimeField', 'DecimalField', 'EmailField', 'Empty', 'Field',
        'FieldDoesNotExist', 'FilePathField', 'FloatField',
        'GenericIPAddressField', 'IPAddressField', 'IntegerField', 'NOT_PROVIDED',
        'NullBooleanField', 'PositiveIntegerField', 'PositiveSmallIntegerField',
        'SlugField', 'SmallIntegerField', 'TextField', 'TimeField', 'URLField',
        """
        
        if field['class'] == 'EmailField':
            return "test%s@test.com" % self.get_iteration_string(iteration)
        elif field['class'] == 'IntegerField':
            return 1
        elif field['class'] == 'CharField':
            return "test%s" % self.get_iteration_string(iteration)
        elif field['class'] == 'DateTimeField':
            value = str(timezone.now().isoformat())
            if value.endswith('+00:00'):
                value = value[:-6] + 'Z'
            return value
        elif field['class'] == 'ForeignKey':
            #We've created fixtures with PK 1 elsewhere
            return 1
        else:
            return None

    


import datetime

from django.template.context import RequestContext
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response, get_list_or_404, get_object_or_404, \
                             redirect, render
from django.utils.safestring import SafeString
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User, Group
from django.utils.translation import ugettext as _  

from rest_framework import filters
from rest_framework import generics,viewsets
from rest_framework import permissions

from rest_framework.authentication import SessionAuthentication, BasicAuthentication
from oauth2_provider.ext.rest_framework import TokenHasReadWriteScope, TokenHasScope

from operis.api.renderers import EmberJSONRenderer 
from operis.api.filters import *
from operis.api import mixins
from operis.api.authentication import KeyAuthentication
{% for animport in imports %}
from {{animport}} import *
{% endfor %}

from {{app}}.django_operis.serializers import *

{% for model in models %}
class {{ model.singular }}ViewSet(viewsets.ModelViewSet):
    #permission_classes = [permissions.IsAuthenticated, TokenHasReadWriteScope]
    permission_classes = [permissions.AllowAny]
    model = {{ model.singular }}
    serializer_class = {{ model.singular }}Serializer
    renderer_classes = (EmberJSONRenderer,)
    filter_backends = (SearchFilter,)
    queryset = {{ model.singular }}.objects.all()
    {{model.filter_fields_converted}}
    {{model.search_fields_converted}}

class {{ model.singular}}View(mixins.ListByIdMixin, generics.RetrieveUpdateDestroyAPIView):
    permission_classes = [permissions.AllowAny]
    model =  {{ model.singular }}
    serializer_class =  {{ model.singular }}Serializer
    renderer_classes = (EmberJSONRenderer,)
{% endfor %}



from django.conf.urls import patterns, url, include
from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.contrib.auth.models import User, Group

admin.autodiscover()

from rest_framework import routers

from {{app}}.django_operis.views import *

# Routers provide an easy way of automatically determining the URL conf
router = routers.DefaultRouter(trailing_slash=False)
{% for model in models -%}
router.register(r'api/v1/{{model.plural_converted}}', {{model.singular}}ViewSet)
{% endfor %}

urlpatterns = patterns('',
     {% for model in models -%}
     url(r'api/v1/{{model.plural_converted}}/(?P<pk>\d+)($|/$)',{{model.singular}}View.as_view()),
     {% endfor %}
     url(r'^', include(router.urls)),
) 



"""
Provides generic filtering backends that can be used to filter the results
returned by list views.
"""
from functools import reduce
import operator

from __future__ import unicode_literals
from django.db import models
from django.http.request import QueryDict
from django.conf import settings   
import django_filters

from rest_framework import filters
from rest_framework.compat import django_filters, six

from operis.utils import unconvert

FilterSet = django_filters and django_filters.FilterSet or None


from datetime import time,tzinfo

from django.conf import settings   
from django.contrib.auth.models import User, Group

from rest_framework import serializers

from operis.api.serializers import *
from operis.api.mixins import *
{% for animport in imports %}
from {{animport}} import *
{% endfor %}
{% for model in models %}
class {{ model.singular }}Serializer(BaseSerializer):
    
    class Meta:
        model = {{ model.singular }}
        resource_name = '{{model.plural_converted}}'
        {{ model.fields_converted }}

{% endfor %}


import datetime                         
import sys
import os.path
import pprint
from inspect import getmembers, isclass
from collections import defaultdict
from optparse import make_option

from django.core.management.base import BaseCommand, CommandError
from django.contrib.contenttypes.models import ContentType
from django.conf import settings
from django.db.models.base import ModelBase

from jinja2 import FileSystemLoader, Environment, PackageLoader, ChoiceLoader

from operis.log import log
from operis.utils import clean, convert, convert_friendly, underscore

class Command(BaseCommand):
    help = 'Creates Generic API Scaffolding' 
    logger = None
    
    option_list = BaseCommand.option_list + (
        make_option('--regenerate',
            action='store_true',
            dest='regenerate',
            default=False,
            help='Wipe Prior instances'),
        )

    def handle(self, *args, **options):
        
        self.logger = log( self )
        
        wipe = False
        if options['regenerate']:
            wipe = True
        
        modules = map(__import__, settings.EMBER_MODELS)
        
        model_history = []
        model_instances = []
        for model in modules:
            for name, obj in getmembers(model.models):
                if isclass(obj):
                    if isinstance(obj, ModelBase):
                        self.logger.log("Object Name is: %s",[obj.__name__],"notice")
                        
                        field_history = []
                        fields_extended = []
                        
                        for aname in obj._meta.get_all_field_names():
                            f, modelItem, direct, m2m = obj._meta.get_field_by_name(aname)
                            #self.logger.log("Field Name :: %s",[f.name],"info")
                            #self.logger.log("Field Class :: %s",[type(f).__name__],"info")
                            try:
                                self.logger.log("Field Type :: %s",[f.get_internal_type()],"debug")
                            except:
                                #for a in dir(f):
                                #    print "%s = %s" % (a,getattr(f,a))
                                continue
                                #sys.exit(0)
                        
                            if hasattr(obj ,"Ember") and hasattr(obj.Ember,'fields'):
                                if f.name not in obj.Ember.fields:
                                    continue
                                    
                            if convert(f.name) not in field_history:
                                if f.name == 'email':
                                    print f
                                field = {}
                                field['name'] = convert(f.name) 
                                field['name_underscore'] = underscore(f.name) 
                                field['name_friendly'] = convert_friendly(f.name)
                                field['type'] = f.get_internal_type()
                                field['class'] = type(f).__name__
                                
                                if field['type'] == "ForeignKey" or field['type'] == "ManyToManyField":
                                    has_parent = True
                                    field['parent'] = underscore(f.rel.to.__name__)
                                    field['parent_class'] = f.rel.to.__name__
                                    
                                field_history.append(convert(f.name))
                                fields_extended.append(field)
                        
                        #print fields_extended
                        
                        index_list = ['id']
                        index_converted = ''
                        
                        fields = ['id']
                        fields_converted = ''
                        
                        filter_fields = ['id']
                        filter_fields_converted = ''
                        
                        search_fields = ['id']
                        search_fields_converted = ''
                        
                        singular = None
                        plural = None
                        
                        if hasattr(obj._meta ,"verbose_name"):
                            singular = unicode(obj._meta.verbose_name)
                        else:
                            singular = obj.__name__
                        
                        if singular in model_history:
                            continue
                            
                        if hasattr(obj._meta ,"verbose_name_plural"):
                            plural = unicode(obj._meta.verbose_name_plural)
                        else:
                            plural = plural.title()
                        
                        #Add to our Plural-Item Controllers
                        if hasattr(obj ,"Ember"):
                            if hasattr(obj.Ember,'fields'):
                                fields = []
                                for f in obj.Ember.fields:
                                    fields.append(convert(f))
                                
                            if hasattr(obj.Ember,'index_list'):
                                index_list = []
                                for f in obj.Ember.index_list:
                                    index_list.append(convert(f))
                            
                            if hasattr(obj.Ember,'filter_fields'):
                                filter_fields = obj.Ember.filter_fields
                                
                            if hasattr(obj.Ember,'search_fields'):
                                search_fields = obj.Ember.search_fields
                                
                        fields_converted = "fields = ('" + "','".join(fields) + "')"
                        index_converted = "fields = ('" + "','".join(index_list) + "')"
                        filter_fields_converted = "filter_fields = ['" + "','".join(filter_fields) + "']"
                        search_fields_converted = "search_fields = ['" + "','".join(search_fields) + "']"
                        
                        item = {    "model": name, 
                                    "singular": clean(singular.title()),
                                    "singular_converted": convert(singular.title()),
                                    "plural": clean(plural),
                                    "plural_converted": convert(plural),
                                    "index_converted": index_converted,
                                    "fields": fields_extended,
                                    "fields_converted": fields_converted,
                                    "filter_fields_converted": filter_fields_converted,
                                    "search_fields_converted": search_fields_converted,
                                }
                        
                        model_history.append(singular)
                                
                        model_instances.append(item)
                        print "=============================="
                        #print obj.__name__
        #sys.exit(0)
                         
        global_exts = getattr(settings, 'JINJA_EXTS', ())
        #env = Environment(extensions=global_exts,loader=FileSystemLoader('templates'))
        env = Environment(extensions=global_exts,loader=PackageLoader('operis','templates'))
        basedir = settings.PROJECT_DIR + "/../" + settings.API_APP_NAME
        
        #Create Operis Subdirectories        
        if not os.path.exists(basedir):
            os.makedirs(basedir)
        if not os.path.exists(basedir+ "/django_operis"):
            os.makedirs(basedir+ "/django_operis")
        if not os.path.isfile(basedir+ "/django_operis/__init__.py"):
            file = open(basedir+ "/django_operis/__init__.py", "w")
            file.write('')
            file.close()                          
        if not os.path.exists(basedir+ "/tests"):
            os.makedirs(basedir+ "/tests")
        if not os.path.isfile(basedir+ "/tests/__init__.py"):
            file = open(basedir+ "/tests/__init__.py", "w")
            file.write('')
            file.close()
                        
        self.logger.log("Directory is %s",[basedir],"notice")
        source = "%s/../../templates/api" % (os.path.dirname(__file__))
        self.logger.log("Source is %s",[source],"notice")
        
        self.logger.log("Creating Base Serializers",[],"success")
        template = env.get_template('api/serializers.py')
        filename = basedir + "/django_operis/serializers.py"
        args = {"app":settings.API_APP_NAME,"imports":settings.EMBER_MODELS,"models":model_instances,"app_name":settings.API_APP_NAME}
        output = template.render(args)
        file = open(filename, "w")
        file.write(output)
        file.close()
        
        self.logger.log("Creating Base Views",[],"success")
        template = env.get_template('api/views.py')
        filename = basedir + "/django_operis/views.py"
        args = {"app":settings.API_APP_NAME,"imports":settings.EMBER_MODELS,"models":model_instances,"app_name":settings.API_APP_NAME}
        output = template.render(args)
        file = open(filename, "w")
        file.write(output)
        file.close()
        
        self.logger.log("Creating Base URLs",[],"success")
        template = env.get_template('api/urls.py')
        filename = basedir + "/django_operis/urls.py"
        args = {"app":settings.API_APP_NAME,"imports":settings.EMBER_MODELS,"models":model_instances,"app_name":settings.API_APP_NAME}
        output = template.render(args)
        file = open(filename, "w")
        file.write(output)
        file.close()
        
        self.logger.log("Creating Base Tests",[],"success")
        template = env.get_template('api/test.py')
        filename = basedir + "/tests/test.py"
        args = {"app":settings.API_APP_NAME,"imports":settings.EMBER_MODELS,"fields":fields, "models":model_instances,"app_name":settings.API_APP_NAME}
        output = template.render(args)
        file = open(filename, "w")
        file.write(output)
        file.close()
        
        self.logger.log("Done, templates are in %s",[settings.EMBER_APP_NAME],"info")
        


class ListByIdMixin(object):
    
    def delete(self, request, *args, **kwargs):
        return self.destroy(request, *args, **kwargs)
        #return Response(status=status.HTTP_204_NO_CONTENT)
        
    def get_queryset(self):
        """
        Get the list of items for this view.
        This must be an iterable, and may be a queryset.
        Defaults to using `self.queryset`.

        You may want to override this if you need to provide different
        querysets depending on the incoming request.

        (Eg. return a list of items that is specific to the user)
        """
        if self.queryset is not None:
            queryset = self.queryset._clone()

        if self.model is not None:
            queryset = self.model._default_manager.all()

        id_values = self.request.QUERY_PARAMS.getlist('ids[]', None)
        if not id_values:
            id_values = self.request.QUERY_PARAMS.getlist('id', None)
        
        if id_values:
            self.paginate_by = None
            queryset = queryset.filter(id__in=id_values)
            if (hasattr(self,"ordering")) and (self.ordering is not None):
                for item in self.ordering:
                    queryset = queryset.order_by(item)

        if queryset is not None:
            return queryset
        
        raise ImproperlyConfigured("'%s' must define 'queryset' or 'model'"
                                    % self.__class__.__name__)
                                    
class ChildrenByIdMixin(object):
    
    def delete(self, request, *args, **kwargs):
        return self.destroy(request, *args, **kwargs)
        #return Response(status=status.HTTP_204_NO_CONTENT)
        
    def get_queryset(self):
        """
        Get the list of items for this view.
        This must be an iterable, and may be a queryset.
        Defaults to using `self.queryset`.

        You may want to override this if you need to provide different
        querysets depending on the incoming request.

        (Eg. return a list of items that is specific to the user)
        """
        if self.queryset is not None:
            queryset = self.queryset._clone()

        if self.model is not None:
            queryset = self.model._default_manager.all()

        print self.kwargs["pk"]
        
        queryset = queryset.filter(ppfa_test__id=self.kwargs["pk"])
        
        if queryset is not None:
            return queryset
        
        raise ImproperlyConfigured("'%s' must define 'queryset' or 'model'"
                                    % self.__class__.__name__)
                                    
    


import json

from django.test import TestCase
from django.test import Client

from autofixture import AutoFixture
from rest_framework import status
from operis.api.test.client import OperisTestClient

{% for animport in imports %}
from {{animport}} import *
{% endfor %}

{% for model in models %}
#http://www.django-rest-framework.org/api-guide/testing/
class {{ model.singular }}Test(TestCase):
    
    def setUp(self):
        # Every test needs a client.
        self.client = OperisTestClient()

    def test_details(self):
        
        {% for field in model.fields -%}
        {% if field.parent_class -%}
        #We create any Foreign Key objects
        #Edit that record
        fixture = AutoFixture({{field.parent_class}})
        fixture.create(1)
        {% endif %}
        {%- endfor %}
        
        # Issue a GET request for NO Records
        list_response = self.client.get('/api/v1/{{ model.plural_converted}}')
        # Check that the response is 200 OK.
        self.assertEqual(list_response.status_code, status.HTTP_200_OK)
        list_data = json.loads(list_response.content)
        # Check that the rendered context contains 5 customers.
        self.assertEqual(len(list_data['{{ model.plural_converted}}']), 0)
        
        test_data = self.client.generate_test_data({{model.fields}})
        
        # Create a record
        post_response = self.client.post('/api/v1/{{ model.plural_converted}}', test_data)
        self.assertEqual(post_response.status_code, status.HTTP_201_CREATED)
        
        # Issue a GET request for ONE Record
        list_response = self.client.get('/api/v1/{{ model.plural_converted}}')
        # Check that the response is 200 OK.
        self.assertEqual(list_response.status_code, status.HTTP_200_OK)
        list_data = json.loads(list_response.content)
        # Check that the rendered context contains 5 customers.
        self.assertEqual(len(list_data['{{ model.plural_converted}}']), 1)
        
        for item in list_data['{{ model.plural_converted}}']:
            #Check for that Record
            item_response = self.client.get('/api/v1/{{ model.plural_converted}}/%s' % item['id'])    
            self.assertEqual(item_response.status_code, status.HTTP_200_OK)
            item_data = json.loads(item_response.content)
            
            self.assertEqual(len(item_data),1)        
            self.assertEqual(item,item_data['{{ model.plural_converted}}'])
            
            test_data_one = self.client.generate_test_data({{model.fields}},1)
            
            {% for field in model.fields -%}
            {% if field.name != 'id' -%}
            #Edit that record
            patch_response = self.client.patch('/api/v1/{{ model.plural_converted}}/%s' % item['id'], {'{{field.name}}': test_data_one['{{field.name}}']})
            self.assertEqual(patch_response.status_code, status.HTTP_200_OK)
            {% endif %}
            {%- endfor %}
            
            #Check for that Record
            item_response = self.client.get('/api/v1/{{ model.plural_converted}}/%s' % item['id'])    
            self.assertEqual(item_response.status_code, status.HTTP_200_OK)
            item_data = json.loads(item_response.content)
            
            self.assertEqual(len(item_data),1)        
            self.assertNotEqual(item,item_data['{{ model.plural_converted}}'])
            {% for field in model.fields -%}
            {% if field.name != 'id' -%}
            self.assertEqual(test_data_one['{{field.name}}'],item_data['{{ model.plural_converted}}']['{{field.name}}']) 
            {% endif %}
            {%- endfor %}
            
            #Delete that record
            patch_response = self.client.delete('/api/v1/{{ model.plural_converted}}/%s' % item['id'])
            self.assertEqual(patch_response.status_code, status.HTTP_204_NO_CONTENT)
           
        # Issue a GET request for NO Records
        list_response = self.client.get('/api/v1/{{ model.plural_converted}}')
        # Check that the response is 200 OK.
        self.assertEqual(list_response.status_code, status.HTTP_200_OK)
        list_data = json.loads(list_response.content)
        # Check that the rendered context contains 5 customers.
        self.assertEqual(len(list_data['{{ model.plural_converted}}']), 0)

{% endfor %}        


import urllib
import urllib2
import json
from math import radians, cos, sin, asin, sqrt

def dictfetchall(cursor):
    "Returns all rows from a cursor as a dict"
    desc = cursor.description
    return [
        dict(zip([col[0] for col in desc], row))
        for row in cursor.fetchall()
    ]
    
def clean_param(param):
    if hasattr(param, '_get_pk_val'):
        # has a pk value -- must be a model
        return str(param._get_pk_val())
    
    if callable(param):
        # it's callable, should call it.
        return str(param())

    return str(param)



"""
Provides generic filtering backends that can be used to filter the results
returned by list views.
"""
from __future__ import unicode_literals
from functools import reduce
import operator

from django.db import models
from django.http.request import QueryDict
from django.conf import settings   
import django_filters

from rest_framework import filters
from rest_framework.compat import django_filters, six

from operis.utils import unconvert

FilterSet = django_filters and django_filters.FilterSet or None

class BaseFilterBackend(object):
    """
    A base class from which all filter backend classes should inherit.
    """

    def filter_queryset(self, request, queryset, view):
        """
        Return a filtered queryset.
        """
        raise NotImplementedError(".filter_queryset() must be overridden.")

class SearchFilter(BaseFilterBackend):
    search_param = 'search'  # The URL query parameter used for the search.

    def get_search_terms(self, request):
        """
        Search terms are set by a ?search=... query parameter,
        and may be comma and/or whitespace delimited.
        """
        params = request.QUERY_PARAMS.get(self.search_param, '')
        return params.replace(',', ' ').split()

    def construct_search(self, field_name):
        if field_name.startswith('^'):
            return "%s__istartswith" % field_name[1:]
        elif field_name.startswith('='):
            return "%s__iexact" % field_name[1:]
        elif field_name.startswith('@'):
            return "%s__search" % field_name[1:]
        else:
            return "%s__icontains" % field_name

    def filter_queryset(self, request, queryset, view):
        search_fields = getattr(view, 'search_fields', None)
        
        new_search_fields = []
        if search_fields:
            for field in search_fields:
                if settings.REST_FRAMEWORK["INITCAP_FIELDS"]:
                    new_search_fields.append(unconvert(field))
                else:
                    new_search_fields.append(field)
        
        
        if not new_search_fields:
            return queryset

        orm_lookups = [self.construct_search(str(search_field))
                       for search_field in new_search_fields]

        for search_term in self.get_search_terms(request):
            or_queries = [models.Q(**{orm_lookup: search_term})
                          for orm_lookup in orm_lookups]
            queryset = queryset.filter(reduce(operator.or_, or_queries))

        return queryset

class OrderingFilter(BaseFilterBackend):
    ordering_param = 'ordering'  # The URL query parameter used for the ordering.

    def get_ordering(self, request):
        """
        Ordering is set by a comma delimited ?ordering=... query parameter.
        """
        params = request.QUERY_PARAMS.get(self.ordering_param)
        if params:
            return [param.strip() for param in params.split(',')]

    def get_default_ordering(self, view):
        ordering = getattr(view, 'ordering', None)
        if isinstance(ordering, six.string_types):
            return (ordering,)
        return ordering

    def remove_invalid_fields(self, queryset, ordering):
        field_names = [field.name for field in queryset.model._meta.fields]
        return [term for term in ordering if term.lstrip('-') in field_names]

    def filter_queryset(self, request, queryset, view):
        ordering = self.get_ordering(request)

        if ordering:
            # Skip any incorrect parameters
            ordering = self.remove_invalid_fields(queryset, ordering)

        if not ordering:
            # Use 'ordering' attribute by default
            ordering = self.get_default_ordering(view)

        if ordering:
            return queryset.order_by(*ordering)

        return queryset

class DjangoFilterBackend(OrderingFilter,BaseFilterBackend):
    """
    A filter backend that uses django-filter.
    """
    default_filter_set = FilterSet

    def __init__(self):
        assert django_filters, 'Using DjangoFilterBackend, but django-filter is not installed'

    def get_filter_class(self, view, queryset=None):
        """
        Return the django-filters `FilterSet` used to filter the queryset.
        """
        filter_class = getattr(view, 'filter_class', None)
        filter_fields = getattr(view, 'filter_fields', None)

        new_filter_fields = []
        if filter_fields:
            for field in filter_fields:
                if settings.REST_FRAMEWORK["INITCAP_FIELDS"]:
                    new_filter_fields.append(unconvert(field))
                else:
                    new_filter_fields.append(field)
        
        if filter_class:
            filter_model = filter_class.Meta.model

            assert issubclass(filter_model, queryset.model), \
                'FilterSet model %s does not match queryset model %s' % \
                (filter_model, queryset.model)

            return filter_class
        
        if new_filter_fields:
            class AutoFilterSet(self.default_filter_set):
                class Meta:
                    model = queryset.model
                    fields = new_filter_fields
            return AutoFilterSet

        return None
    
    def filter_queryset(self, request, queryset, view):
        
        ordering = self.get_ordering(request)

        if ordering:
            # Skip any incorrect parameters
            ordering = self.remove_invalid_fields(queryset, ordering)

        if not ordering:
            # Use 'ordering' attribute by default
            ordering = self.get_default_ordering(view)
        
        if ordering:
            queryset.order_by(*ordering)
        
        filter_class = self.get_filter_class(view, queryset)
        print filter_class
        if filter_class:
            return filter_class(request.QUERY_PARAMS, queryset=queryset).qs
        
        return queryset



from rest_framework import pagination
from rest_framework.response import Response
from rest_framework import serializers

class CustomPaginationSerializer(pagination.PageNumberPagination):
    # Takes the page object as the source
    #meta = CustomMetaSerializer(source='*')
    results_field = 'results'
    
    def get_paginated_response(self, data):
        return Response({
            'meta': {
               'count': self.page.paginator.count,
               'num_pages': self.page.paginator.num_pages,
               'current_page': self.page.number,
               'rpp': self.page.paginator.per_page
            },
            'results': data
        })
        
class BaseSerializer(serializers.ModelSerializer):
    
    pass


import urllib 

from django.contrib.auth.models import User

from rest_framework import authentication
from rest_framework import exceptions

from oauth2_provider.models import *
from oauth2_provider.views.application import ApplicationOwnerIsUserMixin

#http://www.django-rest-framework.org/api-guide/authentication/
class KeyAuthentication(authentication.BaseAuthentication):
    def authenticate(self, request):
        
        model = get_application_model()
        application = None
        
        key = request.META.get('HTTP_AUTHORIZATION')
        
        if 'key' in request.query_params:
            key = request.query_params['key']
            key = urllib.unquote(key)
        
        if key:
            application = model.objects.get(client_id=key)
            
        if not application:
            raise exceptions.AuthenticationFailed('No such key')
        
        try:
            user = application.user
            #user = User.objects.get(pk=1)
        except User.DoesNotExist:
            raise exceptions.AuthenticationFailed('No such user')

        return (user, None)


from rest_framework.renderers import JSONRenderer
from rest_framework.exceptions import ParseError
 
class EmberJSONRenderer(JSONRenderer):
    def render(self, data, accepted_media_type=None, renderer_context=None):
        response_data = {}
        # Name the object list 
        object_list = 'results'
        try:
            meta_dict = getattr(renderer_context.get('view').get_serializer().Meta, 'meta_dict')
        except:
            meta_dict = dict()
        
        resource_name = getattr(renderer_context.get('view').get_serializer().Meta, 'resource_name', 'results')
        meta_dict['resource'] = resource_name
        
        try:
            if (data) and ("results" in data):
                response_data[resource_name] = data['results']
                response_data['meta'] = dict()
                # Add custom meta data
                response_data['meta'].update(meta_dict)
                if "meta" in data:
                    response_data['meta'].update(data['meta'])
                # Call super to render the response
            else:
                response_data[resource_name] = data
        except KeyError:
            raise ParseError()
            
        return super(EmberJSONRenderer, self).render(response_data, accepted_media_type, renderer_context)


from rest_framework import serializers

class CurrentPageField(serializers.ReadOnlyField):
    """
    Field that returns a querystring value.
    """
    
    def to_representation(self, object):
        request = self.context.get('request')
        try:
            return int(request.GET["page"])
        except KeyError:
            return 1
            